# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: hfc/protos/ledger/rwset/rwset.proto

import sys
_b=sys.version_info[0]<3 and (lambda x:x) or (lambda x:x.encode('latin1'))
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
from google.protobuf import descriptor_pb2
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor.FileDescriptor(
  name='hfc/protos/ledger/rwset/rwset.proto',
  package='rwset',
  syntax='proto3',
  serialized_pb=_b('\n#hfc/protos/ledger/rwset/rwset.proto\x12\x05rwset\"\x83\x01\n\x0eTxReadWriteSet\x12\x33\n\ndata_model\x18\x01 \x01(\x0e\x32\x1f.rwset.TxReadWriteSet.DataModel\x12\'\n\x08ns_rwset\x18\x02 \x03(\x0b\x32\x15.rwset.NsReadWriteSet\"\x13\n\tDataModel\x12\x06\n\x02KV\x10\x00\"x\n\x0eNsReadWriteSet\x12\x11\n\tnamespace\x18\x01 \x01(\t\x12\r\n\x05rwset\x18\x02 \x01(\x0c\x12\x44\n\x17\x63ollection_hashed_rwset\x18\x03 \x03(\x0b\x32#.rwset.CollectionHashedReadWriteSet\"e\n\x1c\x43ollectionHashedReadWriteSet\x12\x17\n\x0f\x63ollection_name\x18\x01 \x01(\t\x12\x14\n\x0chashed_rwset\x18\x02 \x01(\x0c\x12\x16\n\x0epvt_rwset_hash\x18\x03 \x01(\x0c\"x\n\x11TxPvtReadWriteSet\x12\x33\n\ndata_model\x18\x01 \x01(\x0e\x32\x1f.rwset.TxReadWriteSet.DataModel\x12.\n\x0cns_pvt_rwset\x18\x02 \x03(\x0b\x32\x18.rwset.NsPvtReadWriteSet\"f\n\x11NsPvtReadWriteSet\x12\x11\n\tnamespace\x18\x01 \x01(\t\x12>\n\x14\x63ollection_pvt_rwset\x18\x02 \x03(\x0b\x32 .rwset.CollectionPvtReadWriteSet\"C\n\x19\x43ollectionPvtReadWriteSet\x12\x17\n\x0f\x63ollection_name\x18\x01 \x01(\t\x12\r\n\x05rwset\x18\x02 \x01(\x0c\x42_\n*org.hyperledger.fabric.protos.ledger.rwsetZ1github.com/hyperledger/fabric/protos/ledger/rwsetb\x06proto3')
)



_TXREADWRITESET_DATAMODEL = _descriptor.EnumDescriptor(
  name='DataModel',
  full_name='rwset.TxReadWriteSet.DataModel',
  filename=None,
  file=DESCRIPTOR,
  values=[
    _descriptor.EnumValueDescriptor(
      name='KV', index=0, number=0,
      options=None,
      type=None),
  ],
  containing_type=None,
  options=None,
  serialized_start=159,
  serialized_end=178,
)
_sym_db.RegisterEnumDescriptor(_TXREADWRITESET_DATAMODEL)


_TXREADWRITESET = _descriptor.Descriptor(
  name='TxReadWriteSet',
  full_name='rwset.TxReadWriteSet',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='data_model', full_name='rwset.TxReadWriteSet.data_model', index=0,
      number=1, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='ns_rwset', full_name='rwset.TxReadWriteSet.ns_rwset', index=1,
      number=2, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
    _TXREADWRITESET_DATAMODEL,
  ],
  options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=47,
  serialized_end=178,
)


_NSREADWRITESET = _descriptor.Descriptor(
  name='NsReadWriteSet',
  full_name='rwset.NsReadWriteSet',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='namespace', full_name='rwset.NsReadWriteSet.namespace', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='rwset', full_name='rwset.NsReadWriteSet.rwset', index=1,
      number=2, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=_b(""),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='collection_hashed_rwset', full_name='rwset.NsReadWriteSet.collection_hashed_rwset', index=2,
      number=3, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=180,
  serialized_end=300,
)


_COLLECTIONHASHEDREADWRITESET = _descriptor.Descriptor(
  name='CollectionHashedReadWriteSet',
  full_name='rwset.CollectionHashedReadWriteSet',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='collection_name', full_name='rwset.CollectionHashedReadWriteSet.collection_name', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='hashed_rwset', full_name='rwset.CollectionHashedReadWriteSet.hashed_rwset', index=1,
      number=2, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=_b(""),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='pvt_rwset_hash', full_name='rwset.CollectionHashedReadWriteSet.pvt_rwset_hash', index=2,
      number=3, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=_b(""),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=302,
  serialized_end=403,
)


_TXPVTREADWRITESET = _descriptor.Descriptor(
  name='TxPvtReadWriteSet',
  full_name='rwset.TxPvtReadWriteSet',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='data_model', full_name='rwset.TxPvtReadWriteSet.data_model', index=0,
      number=1, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='ns_pvt_rwset', full_name='rwset.TxPvtReadWriteSet.ns_pvt_rwset', index=1,
      number=2, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=405,
  serialized_end=525,
)


_NSPVTREADWRITESET = _descriptor.Descriptor(
  name='NsPvtReadWriteSet',
  full_name='rwset.NsPvtReadWriteSet',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='namespace', full_name='rwset.NsPvtReadWriteSet.namespace', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='collection_pvt_rwset', full_name='rwset.NsPvtReadWriteSet.collection_pvt_rwset', index=1,
      number=2, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=527,
  serialized_end=629,
)


_COLLECTIONPVTREADWRITESET = _descriptor.Descriptor(
  name='CollectionPvtReadWriteSet',
  full_name='rwset.CollectionPvtReadWriteSet',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='collection_name', full_name='rwset.CollectionPvtReadWriteSet.collection_name', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='rwset', full_name='rwset.CollectionPvtReadWriteSet.rwset', index=1,
      number=2, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=_b(""),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=631,
  serialized_end=698,
)

_TXREADWRITESET.fields_by_name['data_model'].enum_type = _TXREADWRITESET_DATAMODEL
_TXREADWRITESET.fields_by_name['ns_rwset'].message_type = _NSREADWRITESET
_TXREADWRITESET_DATAMODEL.containing_type = _TXREADWRITESET
_NSREADWRITESET.fields_by_name['collection_hashed_rwset'].message_type = _COLLECTIONHASHEDREADWRITESET
_TXPVTREADWRITESET.fields_by_name['data_model'].enum_type = _TXREADWRITESET_DATAMODEL
_TXPVTREADWRITESET.fields_by_name['ns_pvt_rwset'].message_type = _NSPVTREADWRITESET
_NSPVTREADWRITESET.fields_by_name['collection_pvt_rwset'].message_type = _COLLECTIONPVTREADWRITESET
DESCRIPTOR.message_types_by_name['TxReadWriteSet'] = _TXREADWRITESET
DESCRIPTOR.message_types_by_name['NsReadWriteSet'] = _NSREADWRITESET
DESCRIPTOR.message_types_by_name['CollectionHashedReadWriteSet'] = _COLLECTIONHASHEDREADWRITESET
DESCRIPTOR.message_types_by_name['TxPvtReadWriteSet'] = _TXPVTREADWRITESET
DESCRIPTOR.message_types_by_name['NsPvtReadWriteSet'] = _NSPVTREADWRITESET
DESCRIPTOR.message_types_by_name['CollectionPvtReadWriteSet'] = _COLLECTIONPVTREADWRITESET
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

TxReadWriteSet = _reflection.GeneratedProtocolMessageType('TxReadWriteSet', (_message.Message,), dict(
  DESCRIPTOR = _TXREADWRITESET,
  __module__ = 'hfc.protos.ledger.rwset.rwset_pb2'
  # @@protoc_insertion_point(class_scope:rwset.TxReadWriteSet)
  ))
_sym_db.RegisterMessage(TxReadWriteSet)

NsReadWriteSet = _reflection.GeneratedProtocolMessageType('NsReadWriteSet', (_message.Message,), dict(
  DESCRIPTOR = _NSREADWRITESET,
  __module__ = 'hfc.protos.ledger.rwset.rwset_pb2'
  # @@protoc_insertion_point(class_scope:rwset.NsReadWriteSet)
  ))
_sym_db.RegisterMessage(NsReadWriteSet)

CollectionHashedReadWriteSet = _reflection.GeneratedProtocolMessageType('CollectionHashedReadWriteSet', (_message.Message,), dict(
  DESCRIPTOR = _COLLECTIONHASHEDREADWRITESET,
  __module__ = 'hfc.protos.ledger.rwset.rwset_pb2'
  # @@protoc_insertion_point(class_scope:rwset.CollectionHashedReadWriteSet)
  ))
_sym_db.RegisterMessage(CollectionHashedReadWriteSet)

TxPvtReadWriteSet = _reflection.GeneratedProtocolMessageType('TxPvtReadWriteSet', (_message.Message,), dict(
  DESCRIPTOR = _TXPVTREADWRITESET,
  __module__ = 'hfc.protos.ledger.rwset.rwset_pb2'
  # @@protoc_insertion_point(class_scope:rwset.TxPvtReadWriteSet)
  ))
_sym_db.RegisterMessage(TxPvtReadWriteSet)

NsPvtReadWriteSet = _reflection.GeneratedProtocolMessageType('NsPvtReadWriteSet', (_message.Message,), dict(
  DESCRIPTOR = _NSPVTREADWRITESET,
  __module__ = 'hfc.protos.ledger.rwset.rwset_pb2'
  # @@protoc_insertion_point(class_scope:rwset.NsPvtReadWriteSet)
  ))
_sym_db.RegisterMessage(NsPvtReadWriteSet)

CollectionPvtReadWriteSet = _reflection.GeneratedProtocolMessageType('CollectionPvtReadWriteSet', (_message.Message,), dict(
  DESCRIPTOR = _COLLECTIONPVTREADWRITESET,
  __module__ = 'hfc.protos.ledger.rwset.rwset_pb2'
  # @@protoc_insertion_point(class_scope:rwset.CollectionPvtReadWriteSet)
  ))
_sym_db.RegisterMessage(CollectionPvtReadWriteSet)


DESCRIPTOR.has_options = True
DESCRIPTOR._options = _descriptor._ParseOptions(descriptor_pb2.FileOptions(), _b('\n*org.hyperledger.fabric.protos.ledger.rwsetZ1github.com/hyperledger/fabric/protos/ledger/rwset'))
# @@protoc_insertion_point(module_scope)



import logging

_logger = logging.getLogger(__name__)


class AffiliationService(object):

    def __init__(self, client):
        self._client = client

    def create(self, registrar, name, caname='', force=False):
        """Create a new affiliation.
        The caller must have hf.AffiliationMgr authority.

        If any of the parent affiliations do not exist and 'force' is true,
        create all parent affiliations also.

        :param registrar: Required. The identity of the registrar
         (i.e. who is performing the registration)
        :param name: The affiliation path to create
        :param caname: Optional. Name of the CA to send the request to within
         the Fabric CA server (Default value = '')
        :param force: Optional. (Default value = False)
        :raises RequestException: errors in requests.exceptions
        :raises ValueError: Failed response, json parse error, args missing
        :return: result
        """

        path = 'affiliations'

        if force is True:
            path += '?force=true'

        _logger.debug("create new affiliation with url {0}".format(path))

        req = {
            'name': name,
            'caname': caname
        }

        authorization = self._client.generateAuthToken(req, registrar)
        headers = {'Authorization': authorization}
        verify = self._client._ca_certs_path

        res, st = self._client._send_ca_post(path=path,
                                             json=req,
                                             headers=headers,
                                             verify=verify)

        _logger.debug("Response status {0}".format(st))
        _logger.debug("Raw response json {0}".format(res))

        return res

    def getOne(self, affiliation, registrar):
        """List a specific affiliation at or below the caller's affinity.
        The caller must have hf.AffiliationMgr authority.

        :param affiliation: The affiliation path to be queried.
        :param registrar: Required. The identity of the registrar
         (i.e. who is performing the registration)
        :raises RequestException: errors in requests.exceptions
        :raises ValueError: Failed response, json parse error, args missing
        :return: result
        """

        if not isinstance(affiliation, str):
            raise ValueError('argument "affiliation" is not a valid string')

        path = 'affiliations/' + affiliation
        authorization = self._client.generateAuthToken(None, registrar)
        headers = {'Authorization': authorization}
        verify = self._client._ca_certs_path
        res, st = self._client._send_ca_get(path,
                                            headers=headers,
                                            verify=verify)

        _logger.debug("Response status {0}".format(st))
        _logger.debug("Raw response json {0}".format(res))

        return res

    def getAll(self, registrar):
        """List all affiliations equal to and below the caller's affiliation.
        The caller must have hf.AffiliationMgr authority.

        :param registrar: Required. The identity of the registrar
         (i.e. who is performing the registration)
        :return: result
        :raises RequestException: errors in requests.exceptions
        :raises ValueError: Failed response, json parse error, args missing
        """

        path = 'affiliations'
        authorization = self._client.generateAuthToken(None, registrar)
        headers = {'Authorization': authorization}
        verify = self._client._ca_certs_path
        res, st = self._client._send_ca_get(path,
                                            headers=headers,
                                            verify=verify)

        _logger.debug("Response status {0}".format(st))
        _logger.debug("Raw response json {0}".format(res))

        return res

    def delete(self, affiliation, registrar, force=False):
        """Delete an affiliation.
        The caller must have hf.AffiliationMgr authority.
        Ca server must have cfg.affiliations.allowremove: true

        If force is true and there are any child affiliations or any identities
        are associated with this affiliation or child affiliations, these
         identities and child affiliations
        will be deleted; otherwise, an error is returned.

        :param affiliation: affiliation
        :param registrar: Required. The identity of the registrar
         (i.e. who is performing the registration)
        :param force:  (Default value = False)
        :return: result
        :raises RequestException: errors in requests.exceptions
        :raises ValueError: Failed response, json parse error, args missing
        """

        if not isinstance(affiliation, str):
            raise ValueError('argument "affiliation" is not a valid string')

        path = 'affiliations/' + affiliation
        if force is True:
            path += '?force=true'

        authorization = self._client.generateAuthToken(None, registrar)
        headers = {'Authorization': authorization}
        verify = self._client._ca_certs_path
        res, st = self._client._send_ca_delete(path,
                                               headers=headers,
                                               verify=verify)

        _logger.debug("Response status {0}".format(st))
        _logger.debug("Raw response json {0}".format(res))

        return res

    def update(self, affiliation, registrar, name, caname='',
               force=False):
        """Rename an affiliation.
        The caller must have hf.AffiliationMgr authority.

        If any identities are associated with this affiliation, 'force' is true
        causes these identities' affiliations to be renamed; otherwise, an
        error is returned.

        :param affiliation: The affiliation path to be updated.
        :param registrar: Required. The identity of the registrar
         (i.e. who is performing the registration)
        :param name: The affiliation path to create
        :param caname: Optional. Name of the CA to send the request to within
         the Fabric CA server (Default value = '')
        :param force: Optional. (Default value = False)
        :return: result
        :raises RequestException: errors in requests.exceptions
        :raises ValueError: Failed response, json parse error, args missing
        """

        if not isinstance(affiliation, str):
            raise ValueError('argument "affiliation" is not a valid string')

        if not isinstance(name, str):
            raise ValueError('argument "name" is not a valid string')

        path = 'affiliations/' + affiliation

        if force:
            path += '?force=true'

        req = {
            'name': name
        }
        if caname:
            req['caname'] = caname

        authorization = self._client.generateAuthToken(req, registrar)
        headers = {'Authorization': authorization}
        verify = self._client._ca_certs_path
        res, st = self._client._send_ca_update(path,
                                               json=req,
                                               headers=headers,
                                               verify=verify)

        _logger.debug("Response status {0}".format(st))
        _logger.debug("Raw response json {0}".format(res))

        return res



# Copyright. All Rights Reserved.
#
# SPDX-License-Identifier: Apache-2.0

import logging

from hfc.fabric.user import create_user


_logger = logging.getLogger(__name__ + ".organization")


class Organization(object):
    """ An organization in the network.

    It contains several members.
    """

    def __init__(self, name='org', state_store=None):
        """
        :param name: Name of the organization
        """
        self._name = name
        self._mspid = None
        self._peers = []
        self._orderers = []
        self._CAs = []
        self._state_store = state_store
        self._users = dict()

    def init_with_bundle(self, info):
        """
        Init the peer with given info dict
        :param info: Dict including all info, e.g., endpoint, grpc option
        :return: True or False
        """
        if 'mspid' in info:
            self._mspid = info['mspid']
        if 'peers' in info:
            self._peers = info['peers']
        if 'orderers' in info:
            self._orderers = info['orderers']
        if 'certificateAuthorities' in info:
            self._CAs = info['certificateAuthorities']
        if 'users' in info:
            users = info['users']
            for name in users:
                user = create_user(name, self._name, self._state_store,
                                   self._mspid, users[name].get('private_key'),
                                   users[name].get('cert'))
                self._users[name] = user
        return True

    def get_user(self, name):
        """
        Return user instance with the name.
        :param name: Name of the user
        :return: User instance or None
        """
        if name in self._users:
            return self._users[name]
        return None


def create_org(name, info, state_store):
    """ Factory method to construct an organization instance
    :param name: Name of the organization
    :param info: Info dict for initialization
    :param state_store: State store for data cache
    :return: an organization instance
    """
    org = Organization(name=name, state_store=state_store)
    org.init_with_bundle(info)
    return org



from hfc.fabric_ca.caservice import Enrollment
from hfc.fabric.user import User
from hfc.fabric.user import validate
from hfc.util.crypto.crypto import ecies


class InMemoryWalletStore(object):
    """InMemoryWalletStore stores the identities of users and admins
        in memory
    """

    def __init__(self):
        self._Map = {}

    def exists(self, enrollment_id):
        """Returns whether or not the credentials of a user with a given enrollment_id
            exists in the wallet

        :param enrollment_id: enrollment id
        :return: True or False
        """
        return enrollment_id in self._Map

    def remove(self, enrollment_id):
        """Deletes identities of users with the given user_id

        :param enrollment_id: enrollment id
        :return:
        """
        del self._Map[enrollment_id]

    def put(self, enrollment_id, user_enrollment):
        """Saves the particular Identity in the wallet

        :param enrollment_id: enrollment id
        :param user_enrollment: Enrollment object
        :return:
        """
        if not isinstance(user_enrollment, Enrollment):
            raise ValueError('"user_enrollment" is not a valid Enrollment object')
        self._Map[enrollment_id] = user_enrollment

    def create_user(self, enrollment_id, org, msp_id, state_store=None):
        """Returns an instance of a user whose identity
            is stored in the InMemoryWallet

        :param enrollment_id: enrollment id
        :param org: organization
        :param msp_id: MSP id
        :param state_store: (Default value = None)
        :return: a validated user object
        """
        crypto_suit = ecies()

        if not self.exists(enrollment_id):
            raise AttributeError('"user" does not exist')

        user = User(enrollment_id, org, state_store)
        user.enrollment = self._Map[enrollment_id]
        user.msp_id = msp_id
        user.cryptoSuite = crypto_suit

        return validate(user)



# SPDX-License-Identifier: Apache-2.0

import asyncio
import logging
import copy

from hfc.fabric_network.gateway import Gateway
from hfc.fabric_network.network import Network
from test.integration.utils import BaseTestCase

logger = logging.getLogger(__name__)
logger.setLevel(logging.DEBUG)


class GatewayTest(BaseTestCase):
    def test_gateway(self):
        loop = asyncio.get_event_loop()

        org1 = 'org1.example.com'
        # Channel Create
        response = loop.run_until_complete(
            self.client.channel_create('orderer.example.com',
                                       self.channel_name,
                                       self.user,
                                       config_yaml=self.config_yaml,
                                       channel_profile=self.channel_profile))
        logger.debug(response)
        self.assertTrue(response)

        # Channel join
        channel = self.client.get_channel(self.channel_name)
        self.assertIsNotNone(channel)

        response = loop.run_until_complete(self.client.channel_join(
            requestor=self.user,
            channel_name=self.channel_name,
            peers=['peer0.' + org1, 'peer1.' + org1],
            orderer='orderer.example.com'
        ))
        self.assertTrue(response)

        # Create Gateway connection
        new_gateway = Gateway()

        defaultOptions = dict({
            'top1': {
                'inner11': 10,
                'inner12': 'ten'
                    },
            'top2': {
                'inner21': 20,
                'inner22': 'twenty'
                    }
                })

        overrideOptions = dict({
                    'top1': {
                        'top13': {
                            'inner131': 131
                        }
                    }
                })

        expectedOptions = dict({
                    'top1': {
                        'inner11': 10,
                        'inner12': 'ten',
                        'top13': {
                            'inner131': 131
                        }
                    },
                    'top2': {
                        'inner21': 20,
                        'inner22': 'twenty'
                    }
                })

        currentOptions = copy.deepcopy(defaultOptions)
        result = new_gateway.mergeOptions(currentOptions, overrideOptions)
        self.assertDictEqual(expectedOptions, result)

        overrideOptions = {
                    'top1': {
                        'inner11': 20,
                        'inner12': 'twenty'
                    },
                }
        expectedOptions = {
                        'top1': {
                            'inner11': 20,
                            'inner12': 'twenty'
                        },
                        'top2': {
                            'inner21': 20,
                            'inner22': 'twenty'
                        }
                    }

        currentOptions = copy.deepcopy(defaultOptions)
        result = new_gateway.mergeOptions(currentOptions, overrideOptions)
        self.assertDictEqual(expectedOptions, result)

        overrideOptions = {
                    'top1': {
                        'inner11': 20
                    },
                }
        expectedOptions = {
                    'top1': {
                        'inner11': 20,
                        'inner12': 'ten'
                    },
                    'top2': {
                        'inner21': 20,
                        'inner22': 'twenty'
                    }
                }

        currentOptions = copy.deepcopy(defaultOptions)
        result = new_gateway.mergeOptions(currentOptions, overrideOptions)
        self.assertDictEqual(expectedOptions, result)

        overrideOptions = {
                    'top1': {
                        'inner11': None
                    },
                }
        expectedOptions = {
                    'top1': {
                        'inner11': None,
                        'inner12': 'ten'
                    },
                    'top2': {
                        'inner21': 20,
                        'inner22': 'twenty'
                    }
                }

        currentOptions = copy.deepcopy(defaultOptions)
        result = new_gateway.mergeOptions(currentOptions, overrideOptions)
        self.assertDictEqual(expectedOptions, result)

        overrideOptions = {
                    'single': True
                }
        expectedOptions = {
                    'top1': {
                        'inner11': 10,
                        'inner12': 'ten'
                    },
                    'top2': {
                        'inner21': 20,
                        'inner22': 'twenty'
                    },
                    'single': True
                }

        currentOptions = copy.deepcopy(defaultOptions)
        result = new_gateway.mergeOptions(currentOptions, overrideOptions)
        self.assertDictEqual(expectedOptions, result)

        # TODO: Change this after wallet has been integrated
        options = {'wallet': ''}
        loop.run_until_complete(new_gateway.connect('test/fixtures/network.json', options))
        new_network = loop.run_until_complete(new_gateway.get_network(self.channel_name, self.user))

        self.assertTrue(isinstance(new_network, Network))
        self.assertTrue(self.channel_name in new_gateway.client._channels)



# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
import asyncio
import unittest
from hfc.fabric.client import Client


class ChannelEventHubTest(unittest.TestCase):

    def setUp(self):
        super(ChannelEventHubTest, self).setUp()
        self.client = Client('test/fixtures/network.json')
        self.channel_name = "businesschannel"  # default application channel
        self.channel = self.client.new_channel(self.channel_name)
        self.blocks = []
        self.org = 'org1.example.com'
        self.peer = self.client.get_peer('peer0.' + self.org)
        self.org_admin = self.client.get_user(self.org, 'Admin')

        self.loop = asyncio.get_event_loop()

    def onEvent(self, block):
        self.blocks.append(block)

    def test_start_twice(self):
        channel_event_hub = self.channel.newChannelEventHub(self.peer,
                                                            self.org_admin)

        channel_event_hub.registerBlockEvent(start=0)

        with self.assertRaises(Exception) as e:
            channel_event_hub.connect(start=0)
        self.assertEqual('Not able to connect with start/stop block when a'
                         ' registered listener has those options.',
                         str(e.exception))

    def test_start_twice_from_listener(self):
        channel_event_hub = self.channel.newChannelEventHub(self.peer,
                                                            self.org_admin)

        s = channel_event_hub.connect(start=0, stop='newest')

        with self.assertRaises(Exception) as e:
            channel_event_hub.registerBlockEvent(start=0)

        try:
            self.loop.run_until_complete(s)  # will fail as no peer is running
        except Exception:
            pass

        channel_event_hub.disconnect()
        self.assertEqual('The registration with a start/stop block must be'
                         ' done before calling connect()', str(e.exception))

    def test_registered_before(self):
        channel_event_hub = self.channel.newChannelEventHub(self.peer,
                                                            self.org_admin)

        channel_event_hub.registerChaincodeEvent('foo', 'bar')

        with self.assertRaises(Exception) as e:
            channel_event_hub.registerBlockEvent(start=0)
        self.assertEqual('Only one event registration is allowed when'
                         ' start/stop block are used.',
                         str(e.exception))

    def test_start_bad_connect(self):
        channel_event_hub = self.channel.newChannelEventHub(self.peer,
                                                            self.org_admin)

        with self.assertRaises(Exception) as e:
            channel_event_hub.connect(start='foo')
        self.assertEqual('start value must be: last_seen, oldest, newest or'
                         ' an integer',
                         str(e.exception))

    def test_start_bad_listener(self):
        channel_event_hub = self.channel.newChannelEventHub(self.peer,
                                                            self.org_admin)

        with self.assertRaises(Exception) as e:
            channel_event_hub.registerBlockEvent(start='foo')
        self.assertEqual('start must be an integer',
                         str(e.exception))

    def test_stop_bad_listener(self):
        channel_event_hub = self.channel.newChannelEventHub(self.peer,
                                                            self.org_admin)

        with self.assertRaises(Exception) as e:
            channel_event_hub.registerBlockEvent(stop='foo')
        self.assertEqual('stop must be an integer, newest or sys.maxsize',
                         str(e.exception))

    def test_start_greater_connect(self):
        channel_event_hub = self.channel.newChannelEventHub(self.peer,
                                                            self.org_admin)

        with self.assertRaises(Exception) as e:
            channel_event_hub.connect(start=20, stop=10)
        self.assertEqual('start cannot be greater than stop',
                         str(e.exception))

    def test_start_greater_listener(self):
        channel_event_hub = self.channel.newChannelEventHub(self.peer,
                                                            self.org_admin)

        with self.assertRaises(Exception) as e:
            channel_event_hub.registerBlockEvent(start=20, stop=10)
        self.assertEqual('start cannot be greater than stop',
                         str(e.exception))


if __name__ == '__main__':
    unittest.main()



# Copyright IBM Corp. 2017 All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
import binascii
import logging
import pickle

from cryptography.hazmat.backends import default_backend
from cryptography.hazmat.primitives import serialization
from cryptography.hazmat.primitives.serialization import load_pem_private_key

from hfc.fabric_ca.caservice import Enrollment
from hfc.util.crypto.crypto import ecies

_logger = logging.getLogger(__name__ + ".user")


class User(object):
    """The default implementation of user."""

    def __init__(self, name, org, state_store):
        """Constructor for a user.

        :param name: name
        :param org: org
        :param state_store: persistent state store as a cache
        :return: An instance of user object
        """
        self._name = name
        self._org = org
        self._state_store = state_store
        self._state_store_key = "user." + name + "." + org
        self._roles = []
        self._account = None
        self._affiliation = None
        self._enrollment_secret = None
        self._enrollment = None
        self._msp_id = None
        self._cryptoSuite = None

        user_state = state_store.get_value(self._state_store_key)

        if not user_state:
            self._save_state()
        else:
            self._restore_state()

    @property
    def name(self):
        """Get the user name
        :return: The user name
        """
        return self._name

    @property
    def org(self):
        """Get the org
        :return: The org
        """
        return self._org

    @property
    def roles(self):
        """Get the roles
        :return: The roles
        """
        return self._roles

    @roles.setter
    def roles(self, roles):
        """Set the roles

        :param roles: the roles
        :return:
        """
        self._roles = roles
        self._save_state()

    @property
    def account(self):
        """Get the account
        :return: The account
        """
        return self._account

    @account.setter
    def account(self, account):
        """Set the account

        :param account: the account
        :return:
        """
        self._account = account
        self._save_state()

    @property
    def affiliation(self):
        """Get the affiliation
        :return: The affiliation
        """
        return self._affiliation

    @affiliation.setter
    def affiliation(self, affiliation):
        """Set the affiliation

        :param affiliation: the affiliation
        :return:
        """
        self._affiliation = affiliation
        self._save_state()

    @property
    def enrollment(self):
        """Get the enrollment"""
        return self._enrollment

    @enrollment.setter
    def enrollment(self, enrollment):
        """Set the enrollment

        :param enrollment: the enrollment
        :return:
        """
        self._enrollment = enrollment
        self._save_state()

    @property
    def enrollment_secret(self):
        """Get the enrollment_secret"""
        return self._enrollment_secret

    @enrollment_secret.setter
    def enrollment_secret(self, enrollment_secret):
        """Set the enrollment_secret

        :param enrollment_secret: the enrollment_secret
        :return:
        """
        self._enrollment_secret = enrollment_secret
        self._save_state()

    @property
    def msp_id(self):
        """Get the msp_id"""
        return self._msp_id

    @msp_id.setter
    def msp_id(self, msp_id):
        """Set the msp_id

        :param msp_id: the msp_id
        :return:
        """
        self._msp_id = msp_id
        self._save_state()

    @property
    def cryptoSuite(self):
        """Get the cryptoSuite"""
        return self._cryptoSuite

    @cryptoSuite.setter
    def cryptoSuite(self, cryptoSuite):
        """Set the cryptoSuite

        :param msp_id: the cryptoSuite
        :param cryptoSuite:
        :return:
        """
        self._cryptoSuite = cryptoSuite
        self._save_state()

    def is_registered(self):
        """Check if user registered

        :return: boolean
        """
        return self._enrollment_secret is not None

    def is_enrolled(self):
        """Check if user enrolled

        :return: boolean
        """
        return self._enrollment is not None

    def _save_state(self):
        """Persistent user state."""
        try:
            state = {
                'name': self.name, 'org': self.org, 'roles': self.roles,
                'affiliation': self.affiliation, 'account': self.account,
                'enrollment_secret': self.enrollment_secret,
                'msp_id': self.msp_id
            }

            if self.enrollment:
                enrollment = {
                    'private_key':
                        self.enrollment.private_key.private_bytes(
                            encoding=serialization.Encoding.PEM,
                            format=serialization.PrivateFormat.PKCS8,
                            encryption_algorithm=serialization.NoEncryption()
                        ),
                    'cert': self.enrollment.cert
                }

                state['enrollment'] = enrollment

            self._state_store.set_value(
                self._state_store_key,
                binascii.hexlify(pickle.dumps(state)).decode("utf-8"))
        except Exception as e:
            raise IOError("Cannot serialize the user", e)

    def _restore_state(self):
        """Restore user state."""
        try:
            state = self._state_store.get_value(self._state_store_key)
            state_dict = pickle.loads(
                binascii.unhexlify(state.encode("utf-8")))
            self._name = state_dict['name']
            self.enrollment_secret = state_dict['enrollment_secret']
            enrollment = state_dict['enrollment']
            if enrollment:
                private_key = serialization.load_pem_private_key(
                    enrollment['private_key'],
                    password=None,
                    backend=default_backend()
                )
                cert = enrollment['cert']
                self.enrollment = Enrollment(private_key, cert)
            self.affiliation = state_dict['affiliation']
            self.account = state_dict['account']
            self.roles = state_dict['roles']
            self._org = state_dict['org']
            self.msp_id = state_dict['msp_id']
        except Exception as e:
            raise IOError("Cannot deserialize the user", e)

    def get_attrs(self):
        return ",".join("{}={}"
                        .format(k, getattr(self, k))
                        for k in self.__dict__.keys())

    def __str__(self):
        return "[{}:{}]".format(self.__class__.__name__, self.get_attrs())


def validate(user):
    """Check the user.

    :param user: A user object
    :return: A validated user object
    :raises ValueError: When user property is invalid
    """
    if not user:
        raise ValueError("User cannot be empty.")

    if not user.name:
        raise ValueError("Missing user name.")

    enrollment = user.enrollment
    if not enrollment:
        raise ValueError("Missing user enrollment.")

    if not enrollment.cert:
        raise ValueError("Missing user enrollment cert.")

    if not enrollment.private_key:
        raise ValueError("Missing user enrollment key.")

    if not user.msp_id:
        raise ValueError("Missing msp id.")

    if not user.cryptoSuite:
        raise ValueError("Missing crypto suite.")

    return user


def create_user(name, org, state_store, msp_id, key_path, cert_path,
                crypto_suite=ecies()):
    """Create user

    :param name: user's name
    :param org: org name
    :param state_store: user state store
    :param msp_id: msp id for the user
    :param crypto_suite: the cryptoSuite used to store crypto and key store
         settings (Default value = ecies())
    :param key_path: identity private key path
    :param cert_path: identity public cert path
    :return: a user instance
    """

    _logger.debug("Create user with {}:{}:{}:{}:{}:{}".format(
        name, org, state_store, msp_id, key_path, cert_path
    ))
    with open(key_path, 'rb') as key:
        key_pem = key.read()

    with open(cert_path, 'rb') as cert:
        cert_pem = cert.read()

    private_key = load_pem_private_key(key_pem, None, default_backend())
    enrollment = Enrollment(private_key, cert_pem)

    user = User(name, org, state_store)
    user.enrollment = enrollment
    user.msp_id = msp_id
    user.cryptoSuite = crypto_suite

    return validate(user)



# Copyright 2009-2017 SAP SE or an SAP affiliate company.
# All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
# flake8: noqa

"""Contains the paths and attributes necessary for the integration tests."""
E2E_CONFIG = {
    'test-network': {
        'docker': {
            'compose_file_no_tls': 'test/fixtures/docker-compose-1peer-notls.yaml',
            'compose_file_tls': 'test/fixtures/docker-compose-2orgs-4peers-tls.yaml',
            'compose_file_tls_cli': 'test/fixtures/docker-compose-2orgs-4peers-tls-cli.yaml',
            'compose_file_mutual_tls': 'test/fixtures/docker-compose-2orgs-4peers-mutual-tls.yaml',
        },
        'channel-artifacts': {
            'channel_id': 'businesschannel',
            'channel.tx': 'test/fixtures/e2e_cli/channel-artifacts/channel.tx',
            'config_yaml': 'test/fixtures/e2e_cli/',
            'channel_profile': 'TwoOrgsChannel'
        },
        'orderer': {
            'grpc_endpoint': 'localhost:7050',
            'server_hostname': 'orderer.example.com',
            'tls_cacerts': 'test/fixtures/e2e_cli/crypto-config/ordererOrganizations/'
                           'example.com/tlsca/tlsca.example.com-cert.pem',
            'mspid': 'OrdererMSP',
            'users': {
                'Admin': {
                    'cert': 'Admin@example.com-cert.pem',
                    'private_key': '630e3767a6e1d3c8e646460123d397455103a900efb4d6fb679a9d9c481841fc_sk'}
            }
        },
        'org1.example.com': {
            'mspid': 'Org1MSP',
            'users': {
                'Admin': {
                    'cert': 'Admin@org1.example.com-cert.pem',
                    'private_key': 'c76527489d5820bd04da80a84c07033ca574413f80614091e04f05c276fb6896_sk'
                },
                'User1': {
                    'cert': 'User1@org1.example.com-cert.pem',
                    'private_key': 'da72fd6c0f4595d33eb9ae6f6d06cd171ebc3882fc856960c244b9b5c2b35a90_sk'
                }
            },
            'peers': {
                'peer0': {
                    'grpc_request_endpoint': 'localhost:7051',
                    'grpc_event_endpoint': 'localhost:7053',
                    'server_hostname': 'peer0.org1.example.com',
                    'tls_cacerts': 'test/fixtures/e2e_cli/crypto-config/peerOrganizations/'
                                   'org1.example.com/peers/peer0.org1.example.com/msp/tlscacerts/'
                                   'tlsca.org1.example.com-cert.pem'
                }
            }
        },
        'org2.example.com': {
            'mspid': 'Org2MSP',
            'users': {
                'Admin': {
                    'cert': 'Admin@org2.example.com-cert.pem',
                    'private_key': '7e0b1c172161fe0f33603106935d2584918e12af955108e429dd63d4c043067a_sk'
                },
                'User1': {
                    'cert': 'User1@org2.example.com-cert.pem',
                    'private_key': '73beefad9003c589064deb2128c4f0831ba8003f1233102cc52a188afd05fe61_sk'
                }
            },
            'peers': {
                'peer0': {
                    'grpc_request_endpoint': 'localhost:9051',
                    'grpc_event_endpoint': 'localhost:9053',
                    'server_hostname': 'peer0.org2.example.com',
                    'tls_cacerts': 'test/fixtures/e2e_cli/crypto-config/peerOrganizations/'
                                   'org2.example.com/peers/peer0.org2.example.com/msp/tlscacerts/'
                                   'tlsca.org2.example.com-cert.pem'
                }
            }
        }
    }
}



# SPDX-License-Identifier: Apache-2.0

import os
import unittest
import time

from test.integration.utils import cli_call

from hfc.fabric_ca.caservice import ca_service
from hfc.fabric_network.couchdbwalletstore import CouchDBWalletStore

ENROLLMENT_ID = "admin"
ENROLLMENT_SECRET = "adminpw"
DB_NAME = 'wallet'


class WalletTest(unittest.TestCase):
    def setUp(self):
        self._enrollment_id = ENROLLMENT_ID
        self._enrollment_secret = ENROLLMENT_SECRET
        self._dbName = DB_NAME
        if os.getenv("CA_ADDR"):
            self._ca_server_address = os.getenv("CA_ADDR")
        else:
            self._ca_server_address = "localhost:7054"
        self._compose_file_path = os.path.normpath(
            os.path.join(os.path.dirname(__file__),
                         "../fixtures/ca/docker-compose.yml")
        )
        self._couch_compose_file_path = os.path.normpath(
            os.path.join(os.path.dirname(__file__),
                         "../fixtures/docker-compose-couch.yaml")
        )
        self.start_test_env()

    def tearDown(self):
        self.shutdown_test_env()

    def start_test_env(self):
        cli_call(["docker-compose", "-f", self._compose_file_path, "up", "-d"])
        cli_call(["docker-compose", "-f", self._couch_compose_file_path, "up", "-d"])
        time.sleep(5)

    def shutdown_test_env(self):
        cli_call(["docker-compose", "-f", self._compose_file_path, "down"])
        cli_call(["docker-compose", "-f", self._couch_compose_file_path, "down"])

    def test_enroll(self):
        casvc = ca_service("http://" + self._ca_server_address)
        adminEnrollment = casvc.enroll(self._enrollment_id,
                                       self._enrollment_secret)
        config = 'http://localhost:5984'
        server = CouchDBWalletStore(DB_NAME, config)
        server.put(self._enrollment_id, adminEnrollment)

        self.assertTrue(server.exists(self._enrollment_id))

        server.remove(self._enrollment_id)

        self.assertFalse(server.exists(self._enrollment_id))


if __name__ == '__main__':
    unittest.main()



# Copyright IBM Corp. 2017 All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
import re
import sys
import uuid
import logging
from copy import copy
from hashlib import sha256

from hfc.protos.common import common_pb2
from hfc.protos.common.common_pb2 import BlockMetadataIndex, Status
from hfc.protos.orderer import ab_pb2
from hfc.protos.peer.transaction_pb2 import TxValidationCode
from hfc.protos.utils import create_seek_payload, \
    create_envelope
from hfc.util.utils import current_timestamp, \
    build_header, build_channel_header, pem_to_der
from hfc.fabric.transaction.tx_context import TXContext
from hfc.fabric.transaction.tx_proposal_request import TXProposalRequest
from hfc.fabric.block_decoder import BlockDecoder, FilteredBlockDecoder

_logger = logging.getLogger(__name__ + ".channel_eventhub")

NO_START_STOP = 0
START_ONLY = 1
END_ONLY = 2
START_AND_END = 3


class EventRegistration(object):
    """A class represents event registration."""

    def __init__(self, onEvent=None, unregister=True, disconnect=False):
        self.onEvent = onEvent
        self.unregister = unregister
        self.disconnect = disconnect


class ChaincodeRegistration(object):
    """A class represents chaincode registration."""

    def __init__(self, ccid, pattern, er, as_array):
        self.uuid = uuid.uuid4().hex
        self.ccid = ccid
        self.pattern = pattern
        self.er = er
        self.as_array = as_array


class ChannelEventHub(object):
    """A class represents channel event hub."""

    def __init__(self, peer, channel_name, requestor):
        self._peer = peer
        self._requestor = requestor
        self._channel_name = channel_name

        self.stream = None
        self._start = None
        self._stop = None
        self._filtered = True
        self._as_array = False
        self._reg_nums = []
        self._tx_ids = {}
        self._reg_ids = {}
        self._connected = False
        self._start_stop_action = {}
        self._start_stop_connect = False
        self._last_seen = None
        self._signed_event = None

        self._ending_block_seen = False
        self._ending_block_newest = False

        self._ending_block_newest = False

    @property
    def connected(self):
        """Get the connected
        :return: The connected
        """
        return self._connected

    @connected.setter
    def connected(self, connected):
        """Set the connected

        :param connected: the connected
        :return:
        """
        self._connected = connected

    def _create_seek_info(self, start=None, stop=None):

        behavior = 'BLOCK_UNTIL_READY'

        # build start
        seek_start = ab_pb2.SeekPosition()
        if start is None or start == 'newest':
            seek_start.newest.CopyFrom(ab_pb2.SeekNewest())
        elif start == 'oldest':
            seek_start.oldest.CopyFrom(ab_pb2.SeekOldest())
        else:
            seek_specified_start = ab_pb2.SeekSpecified()
            seek_specified_start.number = start
            seek_start.specified.CopyFrom(seek_specified_start)

        # build stop
        seek_stop = ab_pb2.SeekPosition()
        if stop == 'newest':
            self._ending_block_newest = True
            seek_stop.newest.CopyFrom(ab_pb2.SeekNewest())
            behavior = 'FAIL_IF_NOT_READY'
        elif start == 'oldest':
            seek_stop.oldest.CopyFrom(ab_pb2.SeekOldest())
            behavior = 'FAIL_IF_NOT_READY'
        else:
            seek_specified_stop = ab_pb2.SeekSpecified()
            if stop is not None:
                seek_specified_stop.number = stop
                behavior = 'FAIL_IF_NOT_READY'
            else:
                seek_specified_stop.number = sys.maxsize
            seek_stop.specified.CopyFrom(seek_specified_stop)

        # seek info with all parts
        seek_info = ab_pb2.SeekInfo()
        seek_info.start.CopyFrom(seek_start)
        seek_info.stop.CopyFrom(seek_stop)

        # BLOCK_UNTIL_READY will mean hold the stream open and keep sending
        # as the blocks come in
        # FAIL_IF_NOT_READY will mean if the block is not there throw an error
        seek_info.behavior = ab_pb2.SeekInfo.SeekBehavior.Value(behavior)

        return seek_info

    def _get_stream(self):
        """get the events of the channel.

        :return: the events in success or None in fail.
        """
        _logger.info("create peer delivery stream")

        if self._signed_event is not None:
            return self._peer.delivery(self._signed_event,
                                       filtered=self._filtered)

        seek_info = self._create_seek_info(self._start, self._stop)

        kwargs = {}
        if self._peer._client_cert_path:
            with open(self._peer._client_cert_path, 'rb') as f:
                b64der = pem_to_der(f.read())
                kwargs['tls_cert_hash'] = sha256(b64der).digest()

        tx_context = TXContext(self._requestor, self._requestor.cryptoSuite,
                               TXProposalRequest())

        seek_info_header = build_channel_header(
            common_pb2.HeaderType.Value('DELIVER_SEEK_INFO'),
            tx_context.tx_id,
            self._channel_name,
            current_timestamp(),
            tx_context.epoch,
            **kwargs
        )

        seek_header = build_header(
            tx_context.identity,
            seek_info_header,
            tx_context.nonce)

        seek_payload_bytes = create_seek_payload(seek_header, seek_info)
        sig = tx_context.sign(seek_payload_bytes)
        envelope = create_envelope(sig, seek_payload_bytes)

        # this is a stream response
        return self._peer.delivery(envelope, filtered=self._filtered)

    def check_start_stop_connect(self, start=None, stop=None):
        if start is not None or stop is not None:
            if self._start_stop_action:
                raise Exception('Not able to connect with start/stop block'
                                ' when a registered listener has those options.')

            if start == 'last_seen':
                start = self._last_seen

            if not ((isinstance(start, int)
                     or start in ('oldest', 'newest'))
                    or start is None):
                raise Exception('start value must be: last_seen, oldest, newest or an integer')

            if stop == 'last_seen':
                stop = self._last_seen

            if not ((isinstance(stop, int)
                     or stop == 'newest')
                    or stop is None):
                raise Exception('stop value must be: last_seen, newest, sys.maxsize or an integer')

            if isinstance(start, int) \
                    and isinstance(stop, int)\
                    and start > stop:
                raise Exception('start cannot be greater than stop')

            self._start = start
            self._stop = stop

            self._start_stop_connect = True

    def check_start_stop_listener(self, start=None, stop=None):

        result = NO_START_STOP

        if self._start_stop_action:
            raise Exception('This ChannelEventHub is not open to event'
                            ' listener registrations')

        if start is not None or stop is not None:
            if self.have_registrations():
                raise Exception('Only one event registration is allowed when'
                                ' start/stop block are used.')

            if self._start_stop_connect:
                raise Exception('The registration with a start/stop block'
                                ' must be done before calling connect()')

            if stop is not None:
                if not (isinstance(stop, int)
                        or stop == 'newest'):
                    raise Exception('stop must be an integer, newest or'
                                    ' sys.maxsize')
                else:
                    result = END_ONLY

            if start is not None:
                if not isinstance(start, int):
                    raise Exception('start must be an integer')
                else:
                    # will move result to START_ONLY or START_AND_END
                    result += 1

            if isinstance(start, int) \
                    and isinstance(stop, int) \
                    and start > stop:
                raise Exception('start cannot be greater than stop')

            self._start = start
            self._stop = stop

        return result

    def _processBlockEvents(self, block):
        for reg_num in self._reg_nums:

            if reg_num.unregister:
                self.unregisterBlockEvent(reg_num)

            if reg_num.onEvent is not None:
                reg_num.onEvent(block)

            if reg_num.disconnect:
                self.disconnect()

    def registerBlockEvent(self, unregister=True,
                           start=None, stop=None,
                           disconnect=False, onEvent=None):

        startstop_mode = self.check_start_stop_listener(start, stop)

        reg_num = EventRegistration(onEvent,
                                    unregister=unregister,
                                    disconnect=disconnect)

        def unregister_action():
            return self.unregisterBlockEvent(reg_num)

        self._on_end_actions(reg_num, unregister_action, startstop_mode,
                             unregister, disconnect)

        self._reg_nums.append(reg_num)
        return reg_num

    def unregisterBlockEvent(self, reg_num):
        self._reg_nums.remove(reg_num)

    def handle_filtered_tx(self, block, tx_id, er):
        for ft in block['filtered_transactions']:
            if tx_id == ft['txid'] or tx_id == 'all':

                if er.onEvent is not None:
                    er.onEvent(tx_id, ft['tx_validation_code'],
                               block['number'])
                if er.unregister:
                    self.unregisterTxEvent(tx_id)
                if er.disconnect:
                    self.disconnect()

    def handle_full_tx(self, block, tx_id, er):
        for index, data in enumerate(block['data']['data']):
            channel_header = data['payload']['header']['channel_header']
            if tx_id == channel_header['tx_id'] or tx_id == 'all':

                if er.onEvent is not None:
                    txFilter = BlockMetadataIndex.Value('TRANSACTIONS_FILTER')
                    txStatusCodes = block['metadata']['metadata'][txFilter]
                    status = TxValidationCode.Name(txStatusCodes[index])
                    er.onEvent(tx_id, status, block['header']['number'])
                if er.unregister:
                    self.unregisterTxEvent(tx_id)
                if er.disconnect:
                    self.disconnect()

    def _processTxEvents(self, block):
        for tx_id, er in copy(self._tx_ids).items():
            # filtered block case
            if self._filtered:
                self.handle_filtered_tx(block, tx_id, er)
            else:
                self.handle_full_tx(block, tx_id, er)

    def registerTxEvent(self, tx_id, unregister=None,
                        start=None, stop=None,
                        disconnect=False, onEvent=None):

        startstop_mode = self.check_start_stop_listener(start, stop)

        if tx_id == 'all' and unregister is None:
            unregister = False
        else:
            unregister = True

        er = EventRegistration(onEvent,
                               unregister=unregister,
                               disconnect=disconnect)

        def unregister_action():
            return self.unregisterTxEvent(tx_id)

        self._on_end_actions(er, unregister_action,
                             startstop_mode, unregister, disconnect)

        self._tx_ids[tx_id] = er
        return tx_id

    def unregisterTxEvent(self, tx_id):
        if tx_id in self._tx_ids:
            del self._tx_ids[tx_id]

    def _queue_chaincode_event(self, chaincode_event, block_number,
                               tx_id, tx_status, all_events):

        for ccid in copy(self._reg_ids).keys():
            for cr in self._reg_ids[ccid]:
                if chaincode_event['chaincode_id'] == cr.ccid and \
                        re.match(cr.pattern, chaincode_event['event_name']):

                    evt = {
                        'chaincode_event': chaincode_event,
                        'block_num': block_number,
                        'tx_id': tx_id,
                        'tx_status': tx_status
                    }

                    if ccid not in all_events:
                        all_events[ccid] = [{
                            cr.uuid: {
                                'cr': cr,
                                'evts': [evt]
                            }
                        }]
                    else:
                        for x in all_events[ccid]:
                            _uuid = next(iter(x.keys()))
                            if _uuid == cr.uuid:
                                x[_uuid]['evts'] += [evt]
                                break
                        else:
                            all_events[ccid].append({
                                cr.uuid: {
                                    'cr': cr,
                                    'evts': [evt]
                                }
                            })

    def handle_filtered_chaincode(self, block, all_events):
        for ft in block['filtered_transactions']:
            if 'transaction_actions' in ft:
                tx_actions = ft['transaction_actions']
                for chaincode_action in tx_actions['chaincode_actions']:
                    chaincode_event = chaincode_action['chaincode_event']
                    # need to remove the payload since with filtered blocks it
                    # has an empty byte array value which is not the real value
                    # we do not want the listener to think that is the value
                    del chaincode_event['payload']
                    self._queue_chaincode_event(chaincode_event,
                                                block['number'],
                                                ft['txid'],
                                                ft['tx_validation_code'],
                                                all_events)

    def _handle_full_chaincode(self, tx, block_number, tx_id,
                               tx_status, all_events):
        if 'actions' in tx:
            for t in tx['actions']:
                ppl_r_p = t['payload']['action']['proposal_response_payload']
                chaincode_event = ppl_r_p['extension']['events']
                self._queue_chaincode_event(chaincode_event,
                                            block_number,
                                            tx_id,
                                            tx_status,
                                            all_events)

    def handle_full_chaincode(self, block, all_events):
        if 'data' in block:
            for index, data in enumerate(block['data']['data']):
                payload = data['payload']
                channel_header = payload['header']['channel_header']

                # only ENDORSER_TRANSACTION have chaincode events
                if channel_header['type'] == 3:
                    tx = payload['data']
                    txFilter = BlockMetadataIndex.Value('TRANSACTIONS_FILTER')
                    txStatusCodes = block['metadata']['metadata'][txFilter]
                    tx_status = TxValidationCode.Name(txStatusCodes[index])
                    tx_id = channel_header['tx_id']
                    self._handle_full_chaincode(tx,
                                                block['header']['number'],
                                                tx_id,
                                                tx_status,
                                                all_events)

    def _processChaincodeEvents(self, block):
        if len(self._reg_ids.keys()):
            all_events = {}
            if self._filtered:
                self.handle_filtered_chaincode(block, all_events)
            else:
                self.handle_full_chaincode(block, all_events)

            for events in all_events.values():
                for e in events:
                    for x in e.values():
                        if x['cr'].er.onEvent is not None:
                            if x['cr'].as_array:
                                x['cr'].er.onEvent(x['evts'])
                            else:
                                for e in x['evts']:
                                    x['cr'].er.onEvent(e['chaincode_event'],
                                                       e['block_num'],
                                                       e['tx_id'],
                                                       e['tx_status'])

                        if x['cr'].er.unregister:
                            self.unregisterChaincodeEvent(x['cr'])

                        if x['cr'].er.disconnect:
                            self.disconnect()

    def registerChaincodeEvent(self, ccid, pattern, unregister=False,
                               start=None, stop=None,
                               as_array=None,
                               disconnect=False, onEvent=None):

        startstop_mode = self.check_start_stop_listener(start, stop)

        if as_array is None:
            as_array = self._as_array

        er = EventRegistration(onEvent, unregister=unregister,
                               disconnect=disconnect)
        cr = ChaincodeRegistration(ccid, pattern, er, as_array)

        def unregister_action():
            return self.unregisterChaincodeEvent(cr)

        self._on_end_actions(cr, unregister_action, startstop_mode,
                             unregister, disconnect)

        if ccid in self._reg_ids:
            self._reg_ids[ccid].append(cr)
        else:
            self._reg_ids[ccid] = [cr]
        return cr

    def unregisterChaincodeEvent(self, cr):
        self._reg_ids[cr.ccid].remove(cr)

        if not self._reg_ids[cr.ccid]:
            del self._reg_ids[cr.ccid]

    def have_registrations(self):
        return self._reg_nums != [] \
            or self._tx_ids != {} \
            or self._reg_ids != {}

    def _on_end_actions(self, event_reg, unregister_action, startstop_mode,
                        unregister, disconnect):

        if startstop_mode > 0:
            self._start_stop_action = {
                'event_reg': event_reg,
                'unregister': False,
                'disconnect': False
            }

            _end_register = True
            if unregister is not None:
                _end_register = unregister

            if _end_register and startstop_mode > 1:
                self._start_stop_action['unregister'] = unregister_action

            _end_disconnect = True
            if disconnect is not None:
                _end_disconnect = disconnect

            if _end_disconnect and startstop_mode > 1:
                self._start_stop_action['disconnect'] = True

    def check_replay_end(self):
        if self._stop is not None:
            if isinstance(self._stop, int) and self._stop <= self._last_seen:
                self._ending_block_seen = True
                if self._start_stop_action:
                    if self._start_stop_action['unregister']:
                        self._start_stop_action['unregister']()
                    if self._start_stop_action['disconnect']:
                        self.disconnect()

    async def handle_stream(self, stream):
        async for event in stream:
            if event.WhichOneof('Type') in ('block', 'filtered_block'):
                self.connected = True

                if event.WhichOneof('Type') == 'block':
                    block = BlockDecoder().decode(
                        event.block.SerializeToString())
                    self._last_seen = block['header']['number']
                else:
                    block = FilteredBlockDecoder().decode(
                        event.filtered_block.SerializeToString())
                    self._last_seen = block['number']

                self._processBlockEvents(block)
                self._processTxEvents(block)
                self._processChaincodeEvents(block)

                self.check_replay_end()

            elif event.WhichOneof('Type') == 'status':
                if event.status == Status.Value('SUCCESS'):  # last block
                    if self._ending_block_seen:
                        _logger.debug(f'status received after last block '
                                      f'seen: {event.status}, block-num:'
                                      f' {self._last_seen}')
                    if self._ending_block_newest:
                        self.disconnect()
                else:
                    self.disconnect()
            else:
                _logger.error(f'ChannelEventHub has received a unknown'
                              f' message type {event.WhichOneof("Type")}')

    def connect(self, filtered=True, start=None, stop=None,
                as_array=False,
                target=None, signed_event=None):
        self._filtered = filtered
        self._as_array = as_array

        if target is not None:
            self._peer = target

        if signed_event is not None:
            self._signed_event = signed_event

        self.check_start_stop_connect(start, stop)

        self.stream = self._get_stream()

        return self.handle_stream(self.stream)

    def disconnect(self):
        self.stream.cancel()
        self._start = None
        self._stop = None
        self._filtered = True
        self._peer = None
        self._requestor = None
        self._channel_name = None
        self._start_stop_action = {}
        self._start_stop_connect = False
        self._signed_event = None

        self._ending_block_seen = False
        self._ending_block_newest = False

        self.connected = False



import os
import shutil


from cryptography.hazmat.primitives import serialization

from hfc.fabric_ca.caservice import Enrollment
from hfc.fabric.user import create_user
from hfc.util.keyvaluestore import FileKeyValueStore


class FileSystenWallet(object):
    """FileSystemWallet stores the identities of users and admins
        ie. it contains the Private Key and Enrollment Certificate
    """

    def __init__(self, path=os.getcwd() + '/tmp/hfc-kvs'):
        self._path = path

        os.makedirs(path, exist_ok=True)

    def exists(self, enrollment_id):
        """Returns whether or not the credentials of a user with a given user_id
            exists in the wallet

        :param enrollment_id: enrollment id
        :return: True or False
        """
        return os.path.exists(self._path+'/'+enrollment_id)

    def remove(self, enrollment_id):
        """Deletes identities of users with the given user_id

        :param enrollment_id: enrollment id
        :return:
        """
        dirpath = self._path+'/'+enrollment_id
        if dirpath.exists() and dirpath.is_dir():
            shutil.rmtree(dirpath)

    def create_user(self, enrollment_id, org, msp_id, state_store=None):
        """Returns an instance of a user whose identity
            is stored in the FileSystemWallet

        :param enrollment_id: enrollment id
        :param org: organization
        :param msp_id: MSP id
        :param state_store: state store (Default value = None)
        :return: a user instance
        """
        if not self.exists(enrollment_id):
            raise AttributeError('"user" does not exist')
        state_store = FileKeyValueStore(self._path)
        key_path = self._path + '/' + enrollment_id + '/' + 'private_sk'
        cert_path = self._path + '/' + enrollment_id + '/' + 'enrollmentCert.pem'
        user = create_user(enrollment_id, org, state_store, msp_id, key_path, cert_path)
        return user


class Identity(object):
    """Class represents a tuple containing
        1) enrollment_id
        2) Enrollment Certificate of user
        3) Private Key of user
    """

    def __init__(self, enrollment_id, user):

        if not isinstance(user, Enrollment):
            raise ValueError('"user" is not a valid Enrollment object')

        self._enrollment_id = enrollment_id
        self._EnrollmentCert = user.cert
        self._PrivateKey = user.private_key.private_bytes(encoding=serialization.Encoding.PEM,
                                                          format=serialization.PrivateFormat.PKCS8,
                                                          encryption_algorithm=serialization.NoEncryption())

    def CreateIdentity(self, Wallet):
        """Saves the particular Identity in the wallet

        :param Wallet:
        :return:
        """
        sub_directory = Wallet._path + '/' + self._enrollment_id + '/'
        os.makedirs(sub_directory, exist_ok=True)

        f = open(sub_directory+'private_sk', 'wb')
        f.write(self._PrivateKey)
        f.close()

        f = open(sub_directory+'enrollmentCert.pem', 'wb')
        f.write(self._EnrollmentCert)
        f.close()



# SPDX-License-Identifier: Apache-2.0

import logging

from hfc.fabric import Client
from hfc.fabric_network.network import Network

consoleHandler = logging.StreamHandler()
_logger = logging.getLogger(__name__)

_logger.setLevel(logging.DEBUG)
_logger.addHandler(consoleHandler)


class Gateway(object):
    """The gateway peer provides the connection point for an application to access the Fabric network.
    It can then be connected to a fabric network using the path to network profile.
    """

    def __init__(self):
        """ Construct Gateway. """
        self.client = None
        self.wallet = None
        self.networks = dict()
        self.options = dict()

    def mergeOptions(self, currentOptions, additionalOptions):
        """Merge additional options to current options

        :param currentOptions: current options
        :param additionalOptions: additional options to be merged
        :return: result
        """
        result = currentOptions
        for prop in additionalOptions:
            if prop in result and isinstance(result[prop], dict) and isinstance(additionalOptions[prop], dict):
                self.mergeOptions(result[prop], additionalOptions[prop])
            else:
                result[prop] = additionalOptions[prop]
        return result

    async def connect(self, net_profile, options):
        """
        Connect to the Gateway with a connection profile and connection options.
        :param net_profile: Path to the Connection Profile
        :param options: Options such as wallet identity and user identity
        :return:
        """
        if 'wallet' not in options:
            _logger.error("A wallet must be assigned to a gateway instance")

        if not net_profile:
            self.client = Client()
        else:
            self.client = Client(net_profile=net_profile)

        if 'identity' in options:
            self.current_identity = self.client.get_user(org_name=options['identity']['org_name'],
                                                         name=options['identity']['name'])

    def get_current_identity(self):
        """:return: The current identity being used in the gateway."""
        return self.current_identity

    def get_client(self):
        """:return: Client instance."""
        return self.client

    def get_options(self):
        """:return: the options being used."""
        _logger.debug('in get_options')
        return self.options

    def disconnect(self):
        """Clean up and disconnect this Gateway connection"""
        _logger.debug('in disconnect')
        self.networks.clear()

    # TODO : remove requestor and integrate this with wallet and identity
    async def get_network(self, network_name, requestor):
        """
        Returns an object representing a network
        :param Name of the channel
        :param requestor: User role who issue the request
        :return: Network instance
        """
        method = 'get_network'

        existing_network = self.networks.get(network_name)
        if existing_network:
            _logger.debug('%s - returning existing network:%s', method, network_name)
            return existing_network

        new_network = Network(self, network_name)
        await new_network._initialize({'requestor': requestor})
        self.networks[network_name] = new_network
        return new_network



# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: hfc/protos/orderer/ab.proto

import sys
_b=sys.version_info[0]<3 and (lambda x:x) or (lambda x:x.encode('latin1'))
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from hfc.protos.common import common_pb2 as hfc_dot_protos_dot_common_dot_common__pb2


DESCRIPTOR = _descriptor.FileDescriptor(
  name='hfc/protos/orderer/ab.proto',
  package='orderer',
  syntax='proto3',
  serialized_options=_b('\n%org.hyperledger.fabric.protos.ordererZ,github.com/hyperledger/fabric/protos/orderer'),
  serialized_pb=_b('\n\x1bhfc/protos/orderer/ab.proto\x12\x07orderer\x1a\x1ehfc/protos/common/common.proto\"A\n\x11\x42roadcastResponse\x12\x1e\n\x06status\x18\x01 \x01(\x0e\x32\x0e.common.Status\x12\x0c\n\x04info\x18\x02 \x01(\t\"\x0c\n\nSeekNewest\"\x0c\n\nSeekOldest\"\x1f\n\rSeekSpecified\x12\x0e\n\x06number\x18\x01 \x01(\x04\"\x91\x01\n\x0cSeekPosition\x12%\n\x06newest\x18\x01 \x01(\x0b\x32\x13.orderer.SeekNewestH\x00\x12%\n\x06oldest\x18\x02 \x01(\x0b\x32\x13.orderer.SeekOldestH\x00\x12+\n\tspecified\x18\x03 \x01(\x0b\x32\x16.orderer.SeekSpecifiedH\x00\x42\x06\n\x04Type\"\xc5\x01\n\x08SeekInfo\x12$\n\x05start\x18\x01 \x01(\x0b\x32\x15.orderer.SeekPosition\x12#\n\x04stop\x18\x02 \x01(\x0b\x32\x15.orderer.SeekPosition\x12\x30\n\x08\x62\x65havior\x18\x03 \x01(\x0e\x32\x1e.orderer.SeekInfo.SeekBehavior\"<\n\x0cSeekBehavior\x12\x15\n\x11\x42LOCK_UNTIL_READY\x10\x00\x12\x15\n\x11\x46\x41IL_IF_NOT_READY\x10\x01\"[\n\x0f\x44\x65liverResponse\x12 \n\x06status\x18\x01 \x01(\x0e\x32\x0e.common.StatusH\x00\x12\x1e\n\x05\x62lock\x18\x02 \x01(\x0b\x32\r.common.BlockH\x00\x42\x06\n\x04Type2\x8f\x01\n\x0f\x41tomicBroadcast\x12?\n\tBroadcast\x12\x10.common.Envelope\x1a\x1a.orderer.BroadcastResponse\"\x00(\x01\x30\x01\x12;\n\x07\x44\x65liver\x12\x10.common.Envelope\x1a\x18.orderer.DeliverResponse\"\x00(\x01\x30\x01\x42U\n%org.hyperledger.fabric.protos.ordererZ,github.com/hyperledger/fabric/protos/ordererb\x06proto3')
  ,
  dependencies=[hfc_dot_protos_dot_common_dot_common__pb2.DESCRIPTOR,])



_SEEKINFO_SEEKBEHAVIOR = _descriptor.EnumDescriptor(
  name='SeekBehavior',
  full_name='orderer.SeekInfo.SeekBehavior',
  filename=None,
  file=DESCRIPTOR,
  values=[
    _descriptor.EnumValueDescriptor(
      name='BLOCK_UNTIL_READY', index=0, number=0,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='FAIL_IF_NOT_READY', index=1, number=1,
      serialized_options=None,
      type=None),
  ],
  containing_type=None,
  serialized_options=None,
  serialized_start=486,
  serialized_end=546,
)
_sym_db.RegisterEnumDescriptor(_SEEKINFO_SEEKBEHAVIOR)


_BROADCASTRESPONSE = _descriptor.Descriptor(
  name='BroadcastResponse',
  full_name='orderer.BroadcastResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='status', full_name='orderer.BroadcastResponse.status', index=0,
      number=1, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='info', full_name='orderer.BroadcastResponse.info', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=72,
  serialized_end=137,
)


_SEEKNEWEST = _descriptor.Descriptor(
  name='SeekNewest',
  full_name='orderer.SeekNewest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=139,
  serialized_end=151,
)


_SEEKOLDEST = _descriptor.Descriptor(
  name='SeekOldest',
  full_name='orderer.SeekOldest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=153,
  serialized_end=165,
)


_SEEKSPECIFIED = _descriptor.Descriptor(
  name='SeekSpecified',
  full_name='orderer.SeekSpecified',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='number', full_name='orderer.SeekSpecified.number', index=0,
      number=1, type=4, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=167,
  serialized_end=198,
)


_SEEKPOSITION = _descriptor.Descriptor(
  name='SeekPosition',
  full_name='orderer.SeekPosition',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='newest', full_name='orderer.SeekPosition.newest', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='oldest', full_name='orderer.SeekPosition.oldest', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='specified', full_name='orderer.SeekPosition.specified', index=2,
      number=3, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
    _descriptor.OneofDescriptor(
      name='Type', full_name='orderer.SeekPosition.Type',
      index=0, containing_type=None, fields=[]),
  ],
  serialized_start=201,
  serialized_end=346,
)


_SEEKINFO = _descriptor.Descriptor(
  name='SeekInfo',
  full_name='orderer.SeekInfo',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='start', full_name='orderer.SeekInfo.start', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='stop', full_name='orderer.SeekInfo.stop', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='behavior', full_name='orderer.SeekInfo.behavior', index=2,
      number=3, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
    _SEEKINFO_SEEKBEHAVIOR,
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=349,
  serialized_end=546,
)


_DELIVERRESPONSE = _descriptor.Descriptor(
  name='DeliverResponse',
  full_name='orderer.DeliverResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='status', full_name='orderer.DeliverResponse.status', index=0,
      number=1, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='block', full_name='orderer.DeliverResponse.block', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
    _descriptor.OneofDescriptor(
      name='Type', full_name='orderer.DeliverResponse.Type',
      index=0, containing_type=None, fields=[]),
  ],
  serialized_start=548,
  serialized_end=639,
)

_BROADCASTRESPONSE.fields_by_name['status'].enum_type = hfc_dot_protos_dot_common_dot_common__pb2._STATUS
_SEEKPOSITION.fields_by_name['newest'].message_type = _SEEKNEWEST
_SEEKPOSITION.fields_by_name['oldest'].message_type = _SEEKOLDEST
_SEEKPOSITION.fields_by_name['specified'].message_type = _SEEKSPECIFIED
_SEEKPOSITION.oneofs_by_name['Type'].fields.append(
  _SEEKPOSITION.fields_by_name['newest'])
_SEEKPOSITION.fields_by_name['newest'].containing_oneof = _SEEKPOSITION.oneofs_by_name['Type']
_SEEKPOSITION.oneofs_by_name['Type'].fields.append(
  _SEEKPOSITION.fields_by_name['oldest'])
_SEEKPOSITION.fields_by_name['oldest'].containing_oneof = _SEEKPOSITION.oneofs_by_name['Type']
_SEEKPOSITION.oneofs_by_name['Type'].fields.append(
  _SEEKPOSITION.fields_by_name['specified'])
_SEEKPOSITION.fields_by_name['specified'].containing_oneof = _SEEKPOSITION.oneofs_by_name['Type']
_SEEKINFO.fields_by_name['start'].message_type = _SEEKPOSITION
_SEEKINFO.fields_by_name['stop'].message_type = _SEEKPOSITION
_SEEKINFO.fields_by_name['behavior'].enum_type = _SEEKINFO_SEEKBEHAVIOR
_SEEKINFO_SEEKBEHAVIOR.containing_type = _SEEKINFO
_DELIVERRESPONSE.fields_by_name['status'].enum_type = hfc_dot_protos_dot_common_dot_common__pb2._STATUS
_DELIVERRESPONSE.fields_by_name['block'].message_type = hfc_dot_protos_dot_common_dot_common__pb2._BLOCK
_DELIVERRESPONSE.oneofs_by_name['Type'].fields.append(
  _DELIVERRESPONSE.fields_by_name['status'])
_DELIVERRESPONSE.fields_by_name['status'].containing_oneof = _DELIVERRESPONSE.oneofs_by_name['Type']
_DELIVERRESPONSE.oneofs_by_name['Type'].fields.append(
  _DELIVERRESPONSE.fields_by_name['block'])
_DELIVERRESPONSE.fields_by_name['block'].containing_oneof = _DELIVERRESPONSE.oneofs_by_name['Type']
DESCRIPTOR.message_types_by_name['BroadcastResponse'] = _BROADCASTRESPONSE
DESCRIPTOR.message_types_by_name['SeekNewest'] = _SEEKNEWEST
DESCRIPTOR.message_types_by_name['SeekOldest'] = _SEEKOLDEST
DESCRIPTOR.message_types_by_name['SeekSpecified'] = _SEEKSPECIFIED
DESCRIPTOR.message_types_by_name['SeekPosition'] = _SEEKPOSITION
DESCRIPTOR.message_types_by_name['SeekInfo'] = _SEEKINFO
DESCRIPTOR.message_types_by_name['DeliverResponse'] = _DELIVERRESPONSE
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

BroadcastResponse = _reflection.GeneratedProtocolMessageType('BroadcastResponse', (_message.Message,), dict(
  DESCRIPTOR = _BROADCASTRESPONSE,
  __module__ = 'hfc.protos.orderer.ab_pb2'
  # @@protoc_insertion_point(class_scope:orderer.BroadcastResponse)
  ))
_sym_db.RegisterMessage(BroadcastResponse)

SeekNewest = _reflection.GeneratedProtocolMessageType('SeekNewest', (_message.Message,), dict(
  DESCRIPTOR = _SEEKNEWEST,
  __module__ = 'hfc.protos.orderer.ab_pb2'
  # @@protoc_insertion_point(class_scope:orderer.SeekNewest)
  ))
_sym_db.RegisterMessage(SeekNewest)

SeekOldest = _reflection.GeneratedProtocolMessageType('SeekOldest', (_message.Message,), dict(
  DESCRIPTOR = _SEEKOLDEST,
  __module__ = 'hfc.protos.orderer.ab_pb2'
  # @@protoc_insertion_point(class_scope:orderer.SeekOldest)
  ))
_sym_db.RegisterMessage(SeekOldest)

SeekSpecified = _reflection.GeneratedProtocolMessageType('SeekSpecified', (_message.Message,), dict(
  DESCRIPTOR = _SEEKSPECIFIED,
  __module__ = 'hfc.protos.orderer.ab_pb2'
  # @@protoc_insertion_point(class_scope:orderer.SeekSpecified)
  ))
_sym_db.RegisterMessage(SeekSpecified)

SeekPosition = _reflection.GeneratedProtocolMessageType('SeekPosition', (_message.Message,), dict(
  DESCRIPTOR = _SEEKPOSITION,
  __module__ = 'hfc.protos.orderer.ab_pb2'
  # @@protoc_insertion_point(class_scope:orderer.SeekPosition)
  ))
_sym_db.RegisterMessage(SeekPosition)

SeekInfo = _reflection.GeneratedProtocolMessageType('SeekInfo', (_message.Message,), dict(
  DESCRIPTOR = _SEEKINFO,
  __module__ = 'hfc.protos.orderer.ab_pb2'
  # @@protoc_insertion_point(class_scope:orderer.SeekInfo)
  ))
_sym_db.RegisterMessage(SeekInfo)

DeliverResponse = _reflection.GeneratedProtocolMessageType('DeliverResponse', (_message.Message,), dict(
  DESCRIPTOR = _DELIVERRESPONSE,
  __module__ = 'hfc.protos.orderer.ab_pb2'
  # @@protoc_insertion_point(class_scope:orderer.DeliverResponse)
  ))
_sym_db.RegisterMessage(DeliverResponse)


DESCRIPTOR._options = None

_ATOMICBROADCAST = _descriptor.ServiceDescriptor(
  name='AtomicBroadcast',
  full_name='orderer.AtomicBroadcast',
  file=DESCRIPTOR,
  index=0,
  serialized_options=None,
  serialized_start=642,
  serialized_end=785,
  methods=[
  _descriptor.MethodDescriptor(
    name='Broadcast',
    full_name='orderer.AtomicBroadcast.Broadcast',
    index=0,
    containing_service=None,
    input_type=hfc_dot_protos_dot_common_dot_common__pb2._ENVELOPE,
    output_type=_BROADCASTRESPONSE,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='Deliver',
    full_name='orderer.AtomicBroadcast.Deliver',
    index=1,
    containing_service=None,
    input_type=hfc_dot_protos_dot_common_dot_common__pb2._ENVELOPE,
    output_type=_DELIVERRESPONSE,
    serialized_options=None,
  ),
])
_sym_db.RegisterServiceDescriptor(_ATOMICBROADCAST)

DESCRIPTOR.services_by_name['AtomicBroadcast'] = _ATOMICBROADCAST

# @@protoc_insertion_point(module_scope)



# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: hfc/protos/orderer/cluster.proto

import sys
_b=sys.version_info[0]<3 and (lambda x:x) or (lambda x:x.encode('latin1'))
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from hfc.protos.common import common_pb2 as hfc_dot_protos_dot_common_dot_common__pb2


DESCRIPTOR = _descriptor.FileDescriptor(
  name='hfc/protos/orderer/cluster.proto',
  package='orderer',
  syntax='proto3',
  serialized_options=_b('\n%org.hyperledger.fabric.protos.ordererZ,github.com/hyperledger/fabric/protos/orderer'),
  serialized_pb=_b('\n hfc/protos/orderer/cluster.proto\x12\x07orderer\x1a\x1ehfc/protos/common/common.proto\"/\n\x0bStepRequest\x12\x0f\n\x07\x63hannel\x18\x01 \x01(\t\x12\x0f\n\x07payload\x18\x02 \x01(\x0c\"\x1f\n\x0cStepResponse\x12\x0f\n\x07payload\x18\x01 \x01(\x0c\"`\n\rSubmitRequest\x12\x0f\n\x07\x63hannel\x18\x01 \x01(\t\x12\x1b\n\x13last_validation_seq\x18\x02 \x01(\x04\x12!\n\x07\x63ontent\x18\x03 \x01(\x0b\x32\x10.common.Envelope\">\n\x0eSubmitResponse\x12\x1e\n\x06status\x18\x01 \x01(\x0e\x32\x0e.common.Status\x12\x0c\n\x04info\x18\x02 \x01(\t2}\n\x07\x43luster\x12=\n\x06Submit\x12\x16.orderer.SubmitRequest\x1a\x17.orderer.SubmitResponse(\x01\x30\x01\x12\x33\n\x04Step\x12\x14.orderer.StepRequest\x1a\x15.orderer.StepResponseBU\n%org.hyperledger.fabric.protos.ordererZ,github.com/hyperledger/fabric/protos/ordererb\x06proto3')
  ,
  dependencies=[hfc_dot_protos_dot_common_dot_common__pb2.DESCRIPTOR,])




_STEPREQUEST = _descriptor.Descriptor(
  name='StepRequest',
  full_name='orderer.StepRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='channel', full_name='orderer.StepRequest.channel', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='payload', full_name='orderer.StepRequest.payload', index=1,
      number=2, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=_b(""),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=77,
  serialized_end=124,
)


_STEPRESPONSE = _descriptor.Descriptor(
  name='StepResponse',
  full_name='orderer.StepResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='payload', full_name='orderer.StepResponse.payload', index=0,
      number=1, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=_b(""),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=126,
  serialized_end=157,
)


_SUBMITREQUEST = _descriptor.Descriptor(
  name='SubmitRequest',
  full_name='orderer.SubmitRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='channel', full_name='orderer.SubmitRequest.channel', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='last_validation_seq', full_name='orderer.SubmitRequest.last_validation_seq', index=1,
      number=2, type=4, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='content', full_name='orderer.SubmitRequest.content', index=2,
      number=3, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=159,
  serialized_end=255,
)


_SUBMITRESPONSE = _descriptor.Descriptor(
  name='SubmitResponse',
  full_name='orderer.SubmitResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='status', full_name='orderer.SubmitResponse.status', index=0,
      number=1, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='info', full_name='orderer.SubmitResponse.info', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=257,
  serialized_end=319,
)

_SUBMITREQUEST.fields_by_name['content'].message_type = hfc_dot_protos_dot_common_dot_common__pb2._ENVELOPE
_SUBMITRESPONSE.fields_by_name['status'].enum_type = hfc_dot_protos_dot_common_dot_common__pb2._STATUS
DESCRIPTOR.message_types_by_name['StepRequest'] = _STEPREQUEST
DESCRIPTOR.message_types_by_name['StepResponse'] = _STEPRESPONSE
DESCRIPTOR.message_types_by_name['SubmitRequest'] = _SUBMITREQUEST
DESCRIPTOR.message_types_by_name['SubmitResponse'] = _SUBMITRESPONSE
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

StepRequest = _reflection.GeneratedProtocolMessageType('StepRequest', (_message.Message,), dict(
  DESCRIPTOR = _STEPREQUEST,
  __module__ = 'hfc.protos.orderer.cluster_pb2'
  # @@protoc_insertion_point(class_scope:orderer.StepRequest)
  ))
_sym_db.RegisterMessage(StepRequest)

StepResponse = _reflection.GeneratedProtocolMessageType('StepResponse', (_message.Message,), dict(
  DESCRIPTOR = _STEPRESPONSE,
  __module__ = 'hfc.protos.orderer.cluster_pb2'
  # @@protoc_insertion_point(class_scope:orderer.StepResponse)
  ))
_sym_db.RegisterMessage(StepResponse)

SubmitRequest = _reflection.GeneratedProtocolMessageType('SubmitRequest', (_message.Message,), dict(
  DESCRIPTOR = _SUBMITREQUEST,
  __module__ = 'hfc.protos.orderer.cluster_pb2'
  # @@protoc_insertion_point(class_scope:orderer.SubmitRequest)
  ))
_sym_db.RegisterMessage(SubmitRequest)

SubmitResponse = _reflection.GeneratedProtocolMessageType('SubmitResponse', (_message.Message,), dict(
  DESCRIPTOR = _SUBMITRESPONSE,
  __module__ = 'hfc.protos.orderer.cluster_pb2'
  # @@protoc_insertion_point(class_scope:orderer.SubmitResponse)
  ))
_sym_db.RegisterMessage(SubmitResponse)


DESCRIPTOR._options = None

_CLUSTER = _descriptor.ServiceDescriptor(
  name='Cluster',
  full_name='orderer.Cluster',
  file=DESCRIPTOR,
  index=0,
  serialized_options=None,
  serialized_start=321,
  serialized_end=446,
  methods=[
  _descriptor.MethodDescriptor(
    name='Submit',
    full_name='orderer.Cluster.Submit',
    index=0,
    containing_service=None,
    input_type=_SUBMITREQUEST,
    output_type=_SUBMITRESPONSE,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='Step',
    full_name='orderer.Cluster.Step',
    index=1,
    containing_service=None,
    input_type=_STEPREQUEST,
    output_type=_STEPRESPONSE,
    serialized_options=None,
  ),
])
_sym_db.RegisterServiceDescriptor(_CLUSTER)

DESCRIPTOR.services_by_name['Cluster'] = _CLUSTER

# @@protoc_insertion_point(module_scope)



# Generated by the gRPC Python protocol compiler plugin. DO NOT EDIT!
import grpc

from hfc.protos.gossip import message_pb2 as hfc_dot_protos_dot_gossip_dot_message__pb2


class GossipStub(object):
  """Gossip
  """

  def __init__(self, channel):
    """Constructor.

    Args:
      channel: A grpc.Channel.
    """
    self.GossipStream = channel.stream_stream(
        '/gossip.Gossip/GossipStream',
        request_serializer=hfc_dot_protos_dot_gossip_dot_message__pb2.Envelope.SerializeToString,
        response_deserializer=hfc_dot_protos_dot_gossip_dot_message__pb2.Envelope.FromString,
        )
    self.Ping = channel.unary_unary(
        '/gossip.Gossip/Ping',
        request_serializer=hfc_dot_protos_dot_gossip_dot_message__pb2.Empty.SerializeToString,
        response_deserializer=hfc_dot_protos_dot_gossip_dot_message__pb2.Empty.FromString,
        )


class GossipServicer(object):
  """Gossip
  """

  def GossipStream(self, request_iterator, context):
    """GossipStream is the gRPC stream used for sending and receiving messages
    """
    context.set_code(grpc.StatusCode.UNIMPLEMENTED)
    context.set_details('Method not implemented!')
    raise NotImplementedError('Method not implemented!')

  def Ping(self, request, context):
    """Ping is used to probe a remote peer's aliveness
    """
    context.set_code(grpc.StatusCode.UNIMPLEMENTED)
    context.set_details('Method not implemented!')
    raise NotImplementedError('Method not implemented!')


def add_GossipServicer_to_server(servicer, server):
  rpc_method_handlers = {
      'GossipStream': grpc.stream_stream_rpc_method_handler(
          servicer.GossipStream,
          request_deserializer=hfc_dot_protos_dot_gossip_dot_message__pb2.Envelope.FromString,
          response_serializer=hfc_dot_protos_dot_gossip_dot_message__pb2.Envelope.SerializeToString,
      ),
      'Ping': grpc.unary_unary_rpc_method_handler(
          servicer.Ping,
          request_deserializer=hfc_dot_protos_dot_gossip_dot_message__pb2.Empty.FromString,
          response_serializer=hfc_dot_protos_dot_gossip_dot_message__pb2.Empty.SerializeToString,
      ),
  }
  generic_handler = grpc.method_handlers_generic_handler(
      'gossip.Gossip', rpc_method_handlers)
  server.add_generic_rpc_handlers((generic_handler,))



# SPDX-License-Identifier: Apache-2.0
#
#!/usr/bin/env python
import io
import os
from setuptools import setup, find_packages
from hfc import VERSION

ROOT_DIR = os.path.dirname(__file__)
SOURCE_DIR = os.path.join(ROOT_DIR)

exec(open('hfc/version.py').read())

with open('./requirements.txt') as reqs_txt:
    requirements = [line for line in reqs_txt]

with open('./requirements-test.txt') as test_reqs_txt:
    test_requirements = [line for line in test_reqs_txt]

setup(
    name='fabric-sdk-py',
    version=VERSION,
    keywords=('Hyperledger Fabric', 'SDK'),
    license='Apache License v2.0',
    description="Python SDK for Hyperledger Fabric.",
    long_description=io.open('README.md', encoding='utf-8').read(),
    author='Hyperledger Community',
    url='https://github.com/hyperledger/fabric-sdk-py/',
    download_url='https://github.com/hyperledger/fabric-sdk-py/',
    packages=find_packages(exclude=['docs', 'tests*']),
    platforms='any',
    install_requires=requirements,
    tests_require=test_requirements,
    zip_safe=False,
    test_suite='test',
    classifiers=[
            'Development Status :: 3 - Alpha',
            'Environment :: Other Environment',
            'Intended Audience :: Developers',
            'Operating System :: OS Independent',
            'Programming Language :: Python :: 3.6',
            'Topic :: Utilities',
            'License :: OSI Approved :: Apache Software License',
    ],
    include_package_data=True,
)



# SPDX-License-Identifier: Apache-2.0

import logging

consoleHandler = logging.StreamHandler()
_logger = logging.getLogger(__name__)

_logger.setLevel(logging.DEBUG)
_logger.addHandler(consoleHandler)


class Contract(object):
    """Represents a smart contract (chaincode) instance in a network.
    Applications should get a Contract instance using the
    networks's get_contract method.
    :return: an instance of Contract
    """

    def __init__(self, network, cc_name, gateway):
        self.network = network
        self.channel = network.channel
        self.cc_name = cc_name
        self.gatetway = gateway

    def get_network(self):
        return self.network

    def get_cc_name(self):
        return self.cc_name

    def get_options(self):
        return self.gateway.get_options()

    # TODO: Remove requestor and integrate with wallet from Gateway
    async def submit_transaction(self, name, args, requestor):
        """
        Submit a transaction to the ledger. The transaction function will be
        evaluated on the list of peers discovered and then submitted to the ordering service
        for committing to the ledger.
        """
        channel_name = self.network.channel._name
        cli = self.gatetway.client
        peers = cli._peers

        response = await cli.chaincode_invoke(requestor=requestor,
                                              channel_name=channel_name,
                                              peers=peers,
                                              args=args,
                                              cc_name=self.cc_name,
                                              wait_for_event=True)

        return response

    # TODO: Remove requestor and integrate with wallet from Gateway
    async def evaluate_transaction(self, name, args, requestor):
        """
        Evaluate a transaction function and return its results.
        The transaction function will be evaluated on
        the endorsing peers but the responses will not be sent to
        the ordering service and hence will not be committed to the ledger.
        This is used for querying the world state.
        """
        channel_name = self.network.channel3
        cli = self.gatetway.client
        peers = cli._peers

        response = await cli.chaincode_query(requestor=requestor,
                                             channel_name=channel_name,
                                             peers=peers,
                                             args=args,
                                             cc_name=self.cc_name)
        return response



# Copyright IBM Corp. 2016 All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
import os
import time
import unittest
import random
import string

from hfc.fabric_ca.caservice import CAClient, CAService
from test.integration.utils import cli_call

with open(os.path.join(os.path.dirname(__file__),
                       "../fixtures/ca/enroll-csr.pem")) as f:
    test_pem = f.read()

ENROLLMENT_ID = "admin"
ENROLLMENT_SECRET = "adminpw"


def get_random_username():
    return ''.join(
        [random.choice(string.ascii_letters + string.digits)
         for n in range(9)])


class CATest(unittest.TestCase):
    """Test for ca module. """

    def setUp(self):
        self._enrollment_id = ENROLLMENT_ID
        self._enrollment_secret = ENROLLMENT_SECRET
        if os.getenv("CA_ADDR"):
            self._ca_server_address = os.getenv("CA_ADDR")
        else:
            self._ca_server_address = "localhost:7054"
        self.compose_file_path = os.path.normpath(
            os.path.join(os.path.dirname(__file__),
                         "../fixtures/ca/docker-compose.yml")
        )
        self.start_test_env()

    def tearDown(self):
        self.shutdown_test_env()

    def start_test_env(self):
        cli_call(["docker-compose", "-f", self.compose_file_path, "up", "-d"])
        time.sleep(5)

    def shutdown_test_env(self):
        cli_call(["docker-compose", "-f", self.compose_file_path, "down"])

    def test_get_ca_info(self):
        ca_client = CAClient("http://" + self._ca_server_address)
        ca_chain = ca_client.get_cainfo()
        self.assertTrue(ca_chain.startswith(b"-----BEGIN CERTIFICATE-----"))

    def test_enroll_success(self):
        """Test enroll success.
        """
        ca_client = CAClient("http://" + self._ca_server_address)
        enrollmentCert, caCertChain = ca_client.enroll(self._enrollment_id,
                                                       self._enrollment_secret,
                                                       test_pem)
        self.assertTrue(enrollmentCert
                        .startswith(b"-----BEGIN CERTIFICATE-----"))
        self.assertTrue(caCertChain.startswith(b"-----BEGIN CERTIFICATE-----"))

    def test_enroll_with_generated_csr_success(self):
        """Test enroll with generated csr success.
        """
        ca_service = CAService("http://" + self._ca_server_address)
        enrollment = ca_service.enroll(self._enrollment_id,
                                       self._enrollment_secret)
        self.assertTrue(enrollment.cert
                        .startswith(b"-----BEGIN CERTIFICATE-----"))

    def test_register_success(self):
        """Test register success.
        """
        ca_service = CAService("http://" + self._ca_server_address)
        enrollment = ca_service.enroll(self._enrollment_id,
                                       self._enrollment_secret)
        # use a random username for registering for avoiding already register
        # issues when test suite ran several times
        username = get_random_username()
        secret = enrollment.register(username, 'pass')
        self.assertTrue(secret == 'pass')

    def test_register_without_password_success(self):
        """Test register without password success.
        """
        ca_service = CAService("http://" + self._ca_server_address)
        enrollment = ca_service.enroll(self._enrollment_id,
                                       self._enrollment_secret)
        # use a random username for registering for avoiding already register
        # issues when test suite ran several times
        username = get_random_username()
        secret = enrollment.register(username)
        self.assertTrue(len(secret) == 12)

    def test_already_register(self):
        """Test register a second time.
        """
        ca_service = CAService("http://" + self._ca_server_address)
        enrollment = ca_service.enroll(self._enrollment_id,
                                       self._enrollment_secret)
        # use a random username for registering for avoiding already register
        # issues when test suite ran several times
        username = get_random_username()
        enrollment.register(username)

        # register a second time
        with self.assertRaises(Exception):
            enrollment.register(username)

    def test_revoke_success(self):
        """Test revoke success.
        """
        ca_service = CAService("http://" + self._ca_server_address)
        enrollment = ca_service.enroll(self._enrollment_id,
                                       self._enrollment_secret)
        # use a random username for registering for avoiding already register
        # issues when test suite ran several times
        username = get_random_username()
        secret = enrollment.register(username)

        # enroll new user
        ca_service.enroll(username, secret)

        # now revoke
        RevokedCerts, CRL = enrollment.revoke(username)
        self.assertTrue(CRL == '')
        self.assertTrue(len(RevokedCerts) == 1)
        self.assertTrue('Serial' in RevokedCerts[0])
        self.assertTrue('AKI' in RevokedCerts[0])
        self.assertTrue(len(RevokedCerts[0]['AKI']) > 0)
        self.assertTrue(len(RevokedCerts[0]['Serial']) > 0)

    def test_reenroll_success(self):
        """Test revoke success.
        """
        ca_service = CAService("http://" + self._ca_server_address)
        enrollment = ca_service.enroll(self._enrollment_id,
                                       self._enrollment_secret)
        # use a random username for registering for avoiding already register
        # issues when test suite ran several times
        username = get_random_username()
        secret = enrollment.register(username)

        # enroll new user
        enrollment = ca_service.enroll(username, secret)

        # reenroll
        reenrollment = ca_service.reenroll(enrollment)

        self.assertTrue(reenrollment.cert
                        .startswith(b"-----BEGIN CERTIFICATE-----"))

    def test_reenroll_after_revoke_success(self):
        """Test revoke success.
        """
        ca_service = CAService("http://" + self._ca_server_address)
        enrollment = ca_service.enroll(self._enrollment_id,
                                       self._enrollment_secret)
        # use a random username for registering for avoiding already register
        # issues when test suite ran several times
        username = get_random_username()
        secret = enrollment.register(username)

        # enroll new user
        enrollment = ca_service.enroll(username, secret)

        # now revoke
        enrollment.revoke(username)

        # reenroll
        with self.assertRaises(Exception):
            ca_service.reenroll(enrollment)

    def test_genCRL_success(self):
        """Test revoke success.
        """
        ca_service = CAService("http://" + self._ca_server_address)
        enrollment = ca_service.enroll(self._enrollment_id,
                                       self._enrollment_secret)
        # use a random username for registering for avoiding already register
        # issues when test suite ran several times
        username = get_random_username()
        secret = enrollment.register(username)

        # enroll new user
        ca_service.enroll(username, secret)

        # now revoke
        enrollment.revoke(username)

        # gen CRL
        try:
            enrollment.generateCRL()
        except Exception as e:
            self.fail("generateCRL fails: {0}".format(e))


if __name__ == '__main__':
    unittest.main()



# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: hfc/protos/peer/signed_cc_dep_spec.proto

import sys
_b=sys.version_info[0]<3 and (lambda x:x) or (lambda x:x.encode('latin1'))
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from hfc.protos.peer import proposal_response_pb2 as hfc_dot_protos_dot_peer_dot_proposal__response__pb2


DESCRIPTOR = _descriptor.FileDescriptor(
  name='hfc/protos/peer/signed_cc_dep_spec.proto',
  package='protos',
  syntax='proto3',
  serialized_options=_b('\n\"org.hyperledger.fabric.protos.peerZ)github.com/hyperledger/fabric/protos/peer'),
  serialized_pb=_b('\n(hfc/protos/peer/signed_cc_dep_spec.proto\x12\x06protos\x1a\'hfc/protos/peer/proposal_response.proto\"\x91\x01\n\x1dSignedChaincodeDeploymentSpec\x12!\n\x19\x63haincode_deployment_spec\x18\x01 \x01(\x0c\x12\x1c\n\x14instantiation_policy\x18\x02 \x01(\x0c\x12/\n\x12owner_endorsements\x18\x03 \x03(\x0b\x32\x13.protos.EndorsementBO\n\"org.hyperledger.fabric.protos.peerZ)github.com/hyperledger/fabric/protos/peerb\x06proto3')
  ,
  dependencies=[hfc_dot_protos_dot_peer_dot_proposal__response__pb2.DESCRIPTOR,])




_SIGNEDCHAINCODEDEPLOYMENTSPEC = _descriptor.Descriptor(
  name='SignedChaincodeDeploymentSpec',
  full_name='protos.SignedChaincodeDeploymentSpec',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='chaincode_deployment_spec', full_name='protos.SignedChaincodeDeploymentSpec.chaincode_deployment_spec', index=0,
      number=1, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=_b(""),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='instantiation_policy', full_name='protos.SignedChaincodeDeploymentSpec.instantiation_policy', index=1,
      number=2, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=_b(""),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='owner_endorsements', full_name='protos.SignedChaincodeDeploymentSpec.owner_endorsements', index=2,
      number=3, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=94,
  serialized_end=239,
)

_SIGNEDCHAINCODEDEPLOYMENTSPEC.fields_by_name['owner_endorsements'].message_type = hfc_dot_protos_dot_peer_dot_proposal__response__pb2._ENDORSEMENT
DESCRIPTOR.message_types_by_name['SignedChaincodeDeploymentSpec'] = _SIGNEDCHAINCODEDEPLOYMENTSPEC
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

SignedChaincodeDeploymentSpec = _reflection.GeneratedProtocolMessageType('SignedChaincodeDeploymentSpec', (_message.Message,), dict(
  DESCRIPTOR = _SIGNEDCHAINCODEDEPLOYMENTSPEC,
  __module__ = 'hfc.protos.peer.signed_cc_dep_spec_pb2'
  # @@protoc_insertion_point(class_scope:protos.SignedChaincodeDeploymentSpec)
  ))
_sym_db.RegisterMessage(SignedChaincodeDeploymentSpec)


DESCRIPTOR._options = None
# @@protoc_insertion_point(module_scope)



import logging
import urllib.parse

_logger = logging.getLogger(__name__)


class CertificateService(object):

    def __init__(self, client):

        if not client:
            raise Exception('Missing Required Argument client<FabricCAClient>')

        self.client = client

    def getCertificates(self, registrar, id=None, aki=None, serial=None,
                        revoked_start=None, revoked_end=None,
                        expired_start=None, expired_end=None,
                        notexpired=None, notrevoked=None, ca=None):
        """The caller will be able to view certificates that it owns. In addition,
        if the caller has **hf.Registrar.Roles** or **hf.Revoker** attribute,
        it will be able to view certificates for identities that have
         affiliations equal to or below the caller's affiliation.

        :param registrar: Required. The identity of the registrar
         (i.e. who is performing the revocation) signing certificate, hash
          algorithm and signature algorithm
        :param id: The enrollment ID that uniquely identifies an identity (Default value = None)
        :param aki: Authority Key Identifier string, hex encoded, for the
          specific certificate (Default value = None)
        :param serial: The serial number for a certificate (Default value = None)
        :param revoked_start: Get revoked certificates starting at the specified time,
         either as timestamp (RFC3339 format) or duration (-30d) (Default value = None)
        :param revoked_end: Get revoked certificates before the specified time,
         either as timestamp * (RFC3339 format) or duration (-15d) (Default value = None)
        :param expired_start: Get expired certificates starting at the
         specified time, either as timestamp (RFC3339 format) or duration (-30d) (Default value = None)
        :param expired_end:  Get expired certificates before the specified time,
         either as timestamp (RFC3339 format) or duration (-15d) (Default value = None)
        :param notexpired: Don't return expired certificates (Default value = None)
        :param notrevoked: Don't return revoked certificates (Default value = None)
        :param ca: The name of the CA to direct this request to within the
         server, or the default CA if not specified (Default value = None)
        :return: result
        :raises RequestException: errors in requests.exceptions
        :raises ValueError: Failed response, json parse error, args missing
        """

        path = 'certificates'

        req = {}
        if id and isinstance(id, str):
            req['id'] = id
        if aki and isinstance(aki, str):
            req['aki'] = aki
        if serial and isinstance(serial, str):
            req['serial'] = serial
        if revoked_start and isinstance(revoked_start, str):
            req['revoked_start'] = revoked_start
        if revoked_end and isinstance(revoked_end, str):
            req['revoked_end'] = revoked_end
        if expired_start and isinstance(expired_start, str):
            req['expired_start'] = expired_start
        if expired_end and isinstance(expired_end, str):
            req['expired_end'] = expired_end
        if notrevoked and isinstance(notrevoked, bool):
            req['notrevoked'] = notrevoked
        if notexpired and isinstance(notexpired, bool):
            req['notexpired'] = notexpired
        if ca and isinstance(ca, str):
            req['ca'] = ca

        queryString = urllib.parse.quote_plus(urllib.parse.urlencode(req))
        if queryString:
            path += '?' + queryString

        authorization = self.client.generateAuthToken(None, registrar)
        headers = {'Authorization': authorization}
        verify = self.client._ca_certs_path

        _logger.debug('getCertificates with url: {0}'.format(path))
        res, st = self.client._send_ca_get(path,
                                           headers=headers,
                                           verify=verify)

        _logger.debug('Response status {0}'.format(st))
        _logger.debug('Raw response json {0}'.format(res))

        return res
